@@ -3,8 +3,9 @@ from pyctp import saftvrmie
 import scipy.linalg as lin
 from scipy.constants import Boltzmann, Avogadro
 from scipy.integrate import quad
-from pykingas.KineticGas import cpp_KineticGas
-import warnings
+from pykingas import cpp_KineticGas, bcolors, suppress_stdout
+from pykingas.OmegaDb import OmegaDb
+import warnings, sys, time, atexit
 
 FLT_EPS = 1e-12
 
@@ -12,6 +13,8 @@ def check_valid_composition(x):
     if abs(sum(x) - 1) > FLT_EPS:
         warnings.warn('Mole fractions do not sum to unity, sum(x) = '+str(sum(x)))
 
+potential_mode_map = {'hs' : 0, 'mie' : 1} # Map string identifiers to corresponding int indentifiers used on cpp-side
+
 class KineticGas:
 
     default_N = 4
@@ -19,7 +22,8 @@ class KineticGas:
     def __init__(self, comps,
                  mole_weights=None, sigma=None, eps_div_k=None,
                  la=None, lr=None, lij=0, kij=0,
-                 BH=False, hs_mixing_rule='additive'):
+                 BH=False, hs_mixing_rule='additive',
+                 potential='HS'):
         '''
         :param comps (str): Comma-separated list of components, following Thermopack-convention
         :param BH (bool) : Use Barker-Henderson diameters?
@@ -36,11 +40,16 @@ class KineticGas:
         :param hs_mixing_rule : If "additive", sigma_12 = (1 - lij) * 0.5 * (sigma_1 + sigma_2),
                                 else: Compute sigma_12 from BH using epsilon_12 and additive sigma_12
                                 Only applicable if BH is True
+        :param potential_mode (str) : What potential to use for collision integrals. Options are
+                                        'HS' : Use hard-sphere potential
+                                        'Mie' : Use Mie-potential
         '''
         if len(comps.split(',')) > 2:
             raise IndexError('Current implementation is only binary-compatible!')
-
+        self.comps = comps
+        self.__omega_db = OmegaDb(comps)
         self.BH = BH
+        self.__potential_mode = potential.lower()
         self.computed_d_points = {} # dict of state points in which (d_1, d0, d1) have already been computed
         self.computed_a_points = {}  # dict of state points in which (a_1, a1) have already been computed
 
@@ -79,17 +88,60 @@ class KineticGas:
                                                       # because a Temperature must be supplied to compute BH-diameter
         self.sigma = np.diag(self.sigma_ij)
 
-        self.cpp_kingas = cpp_KineticGas(self.mole_weights, self.sigma_ij)
+        if self.__potential_mode == 'mie':
+            omegapoints, omegavals = self.__omega_db.db_to_vectors()
+        else:
+            omegapoints = [] # 2d array with shape (0, 0)
+            omegavals = [] # 1d array with shape (0, )
 
-    def get_A_matrix(self, T, mole_fracs, N=default_N):
+        self.cpp_kingas = cpp_KineticGas(self.mole_weights, self.sigma_ij, self.epsilon_ij, self.la, self.lr,
+                                         potential_mode_map[self.__potential_mode], omegapoints, omegavals)
+
+        self.cpp_kingas_BH = cpp_KineticGas(self.mole_weights, self.sigma_ij, self.epsilon_ij, self.la, self.lr,
+                                            potential_mode_map[self.__potential_mode])
+
+        if self.__potential_mode == 'mie':
+            atexit.register(self.update_omega_db)
+
+    def update_omega_db(self):
+        self.__omega_db.update(self.cpp_kingas.omega_map)
+        self.__omega_db.dump()
+
+    def get_A_matrix(self, T, mole_fracs, BH=False, N=default_N):
         # Compute the matrix of a_pq values
         check_valid_composition(mole_fracs)
-        return self.cpp_kingas.get_A_matrix(T, mole_fracs, N)
+        if BH is True:
+            sigmaij = self.get_sigma_matrix(self.sigma, BH=BH, T=T)
+            self.cpp_kingas_BH = cpp_KineticGas(self.mole_weights, sigmaij, self.epsilon_ij, self.la, self.lr, potential_mode_map[self.__potential_mode])
+            return self.cpp_kingas_BH.get_A_matrix(T, mole_fracs, N)
+        else:
+            return self.cpp_kingas.get_A_matrix(T, mole_fracs, N)
 
-    def get_reduced_A_matrix(self, T, mole_fracs, N=default_N):
+    def get_delta_vector(self, T, particle_density, BH=False, N=default_N):
+        if BH is True:
+            sigmaij = self.get_sigma_matrix(self.sigma, BH=BH, T=T)
+            self.cpp_kingas_BH = cpp_KineticGas(self.mole_weights, sigmaij, self.epsilon_ij, self.la, self.lr, potential_mode_map[self.__potential_mode])
+            return self.cpp_kingas_BH.get_delta_vector(T, particle_density, N)
+        else:
+            return self.cpp_kingas.get_delta_vector(T, particle_density, N)
+
+    def get_reduced_A_matrix(self, T, mole_fracs, BH=False, N=default_N):
         # Compute the matrix of a_pq values, without a_0q and a_p0
         check_valid_composition(mole_fracs)
-        return self.cpp_kingas.get_reduced_A_matrix(T, mole_fracs, N)
+        if BH is True:
+            sigmaij = self.get_sigma_matrix(self.sigma, BH=BH, T=T)
+            self.cpp_kingas_BH = cpp_KineticGas(self.mole_weights, sigmaij, self.epsilon_ij, self.la, self.lr, potential_mode_map[self.__potential_mode])
+            return self.cpp_kingas_BH.get_reduced_A_matrix(T, mole_fracs, N)
+        else:
+            return self.cpp_kingas.get_reduced_A_matrix(T, mole_fracs, N)
+
+    def get_alpha_vector(self, T, particle_density, mole_fracs, BH=False, N=default_N):
+        if BH is True:
+            sigmaij = self.get_sigma_matrix(self.sigma, BH=BH, T=T)
+            self.cpp_kingas_BH = cpp_KineticGas(self.mole_weights, sigmaij, self.epsilon_ij, self.la, self.lr, potential_mode_map[self.__potential_mode])
+            return self.cpp_kingas_BH.get_alpha_vector(T, particle_density, mole_fracs, N)
+        else:
+            return self.cpp_kingas.get_alpha_vector(T, particle_density, mole_fracs, N)
 
     def compute_d_vector(self, T, particle_density, mole_fracs, N=default_N, BH=False):
         # Compute (d_{-1}, d_0 and d_1), used in diifusion solutions
@@ -97,16 +149,8 @@ class KineticGas:
         if (T, particle_density, tuple(mole_fracs), N, BH) in self.computed_d_points.keys():
             return self.computed_d_points[(T, particle_density, tuple(mole_fracs), N, BH)]
 
-        if BH:
-            sigmaij = self.get_sigma_matrix(self.sigma, BH=BH, T=T)
-            cpp_kingas = cpp_KineticGas(self.mole_weights, sigmaij)
-
-        else:
-            cpp_kingas = self.cpp_kingas
-
-        A = cpp_kingas.get_A_matrix(T, mole_fracs, N)
-
-        delta = cpp_kingas.get_delta_vector(T, particle_density, N)
+        A = self.get_A_matrix(T, mole_fracs, BH=BH, N=N)
+        delta = self.get_delta_vector(T, particle_density, BH=BH, N=N)
         d = lin.solve(A, delta)
         d_1, d0, d1 = d[N - 1], d[N], d[N + 1]
 
@@ -119,14 +163,9 @@ class KineticGas:
         if (T, particle_density, tuple(mole_fracs), N, BH) in self.computed_a_points.keys():
             return self.computed_a_points[(T, particle_density, tuple(mole_fracs), N, BH)]
 
-        if BH:
-            sigmaij = self.get_sigma_matrix(self.sigma, BH=BH, T=T)
-            cpp_kingas = cpp_KineticGas(self.mole_weights, sigmaij)
-        else:
-            cpp_kingas = self.cpp_kingas
+        A = self.get_reduced_A_matrix(T, mole_fracs, BH=BH, N=N)
+        alpha = self.get_alpha_vector(T, particle_density, mole_fracs, BH=BH, N=N)
 
-        A = cpp_kingas.get_reduced_A_matrix(T, mole_fracs, N)
-        alpha = cpp_kingas.get_alpha_vector(T, particle_density, mole_fracs, N)
         a = lin.solve(A, alpha)
         a_1, a1 = a[N - 1], a[N]
 
@@ -144,6 +183,10 @@ class KineticGas:
         kT_vec = np.array([kT, -kT])
         return kT_vec * ((1 / np.array(x)) + (1 / (1 - np.array(x))) )
 
+    def soret(self, T, Vm, x, N=default_N, BH=None):
+        # Compute the Soret coefficient
+        return self.alpha_T0(T, Vm, x, N=N, BH=BH) / T
+
     def interdiffusion(self, T, Vm, x, N=default_N, BH=None):
         if BH is None:
             BH = self.BH
@@ -216,31 +259,4 @@ class KineticGas:
         l = np.array(lambdas)
         return 3 + np.sqrt((l - 3) * np.vstack(l - 3))
 
-def test():
-    comps = 'AR,HE'
-    kingas = KineticGas(comps)
 
-    T = 300
-    x = [0.7, 0.3]
-    Vm = 24e-3
-
-    alpha_T0 = kingas.alpha_T0(T, Vm, x)
-    D12 = kingas.interdiffusion(T, Vm, x)
-    DT = kingas.thermal_diffusion(T, Vm, x)
-    thermal_cond = kingas.thermal_conductivity(T, Vm, x)
-
-    kingas.alpha_T0(T, Vm, x, BH=True)
-    kingas.interdiffusion(T, Vm, x, BH=True)
-    kingas.thermal_diffusion(T, Vm, x, BH=True)
-    kingas.thermal_conductivity(T, Vm, x, BH=True)
-
-    if any(abs(alpha_T0 - kingas.alpha_T0(T, Vm, x)) > FLT_EPS):
-        return 1
-    if abs(D12 - kingas.interdiffusion(T, Vm, x)) > FLT_EPS:
-        return 2
-    if abs(DT - kingas.thermal_diffusion(T, Vm, x)) > FLT_EPS:
-        return 3
-    if abs(thermal_cond - kingas.thermal_conductivity(T, Vm, x)) > FLT_EPS:
-        return 4
-
-    return 0
